@@ -1,6 +1,6 @@
 /*
-** Copyright (C) 2009-2016 Quadrant Information Security <quadrantsec.com>
-** Copyright (C) 2009-2016 Champ Clark III <cclark@quadrantsec.com>
+** Copyright (C) 2009-2017 Quadrant Information Security <quadrantsec.com>
+** Copyright (C) 2009-2017 Champ Clark III <cclark@quadrantsec.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
@@ -22,15 +22,15 @@
 #include "config.h"             /* From autoconf */
 #endif
 
-#include "parsers/sagan-strstr/sagan-strstr-hook.h"
+#include "parsers/strstr-asm/strstr-hook.h"
 
-char *Sagan_Parse_IP( char *, int );
-int   Sagan_Parse_Src_Port( char * );
-int   Sagan_Parse_Dst_Port( char * );
-int   Sagan_Parse_Proto( char * );
-int   Sagan_Parse_Proto_Program( char * );
-char *Sagan_Parse_Hash(char *, int );
-char *Sagan_Parse_Hash_Cleanup(char *);
+void  Parse_IP( char *, int, char *str, size_t size );
+int   Parse_Src_Port( char * );
+int   Parse_Dst_Port( char * );
+int   Parse_Proto( char * );
+int   Parse_Proto_Program( char * );
+void  Parse_Hash( char *, int, char *str, size_t size );
+void  Parse_Hash_Cleanup(char *, char *str, size_t size );
 
 
 
