@@ -1,6 +1,6 @@
 /*
-** Copyright (C) 2009-2016 Quadrant Information Security <quadrantsec.com>
-** Copyright (C) 2009-2016 Champ Clark III <cclark@quadrantsec.com>
+** Copyright (C) 2009-2017 Quadrant Information Security <quadrantsec.com>
+** Copyright (C) 2009-2017 Champ Clark III <cclark@quadrantsec.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
@@ -50,64 +50,78 @@
 #include <netinet/in.h>
 #include <sys/socket.h>
 #include <stdbool.h>
+#include <sys/wait.h>
 
 #include "sagan.h"
 #include "sagan-defs.h"
 #include "version.h"
 
-#include "sagan-credits.h"
-#include "sagan-flowbit.h"
-#include "sagan-processor.h"
+#include "credits.h"
+#include "xbit-mmap.h"
+#include "processor.h"
 #include "sagan-config.h"
-#include "sagan-ignore-list.h"
-#include "sagan-key.h"
-#include "sagan-lockfile.h"
-#include "sagan-signal.h"
-#include "sagan-usage.h"
-#include "sagan-stats.h"
-#include "sagan-ipc.h"
+#include "config-yaml.h"
+#include "ignore-list.h"
+#include "key.h"
+#include "lockfile.h"
+#include "signal-handler.h"
+#include "usage.h"
+#include "stats.h"
+#include "ipc.h"
 #include "parsers/parsers.h"
 
-#ifdef HAVE_LIBPCAP
-#include "sagan-plog.h"
+#ifdef HAVE_SYS_PRCTL_H
+#include <sys/prctl.h>
 #endif
 
-#include "processors/sagan-engine.h"
-#include "processors/sagan-blacklist.h"
-#include "processors/sagan-track-clients.h"
-#include "processors/sagan-report-clients.h"
-#include "processors/sagan-perfmon.h"
-#include "processors/sagan-bro-intel.h"
+#ifdef HAVE_LIBPCAP
+#include "plog.h"
+#endif
+
+#include "processors/engine.h"
+#include "processors/blacklist.h"
+#include "processors/track-clients.h"
+#include "processors/perfmon.h"
+#include "processors/bro-intel.h"
 
 #ifdef HAVE_LIBLOGNORM
-#include "sagan-liblognorm.h"
+#include "liblognormalize.h"
 #endif
 
 #if defined(HAVE_DNET_H) || defined(HAVE_DUMBNET_H)
-#include "output-plugins/sagan-unified2.h"
+#include "output-plugins/unified2.h"
 #endif
 
 #define OVECCOUNT 30
 
 struct _SaganCounters *counters;
 struct _Rule_Struct *rulestruct;
-struct class_struct *classstruct;
 struct _SaganConfig *config;
 struct _SaganDebug *debug;
-struct _Sagan_Flowbit *flowbit;
 
 #ifdef WITH_BLUEDOT
 #include <curl/curl.h>
-#include "processors/sagan-bluedot.h"
+#include "processors/bluedot.h"
+#endif
+
+#ifdef HAVE_LIBHIREDIS
+#include <hiredis/hiredis.h>
+#include "redis.h"
 #endif
 
 struct _Sagan_Proc_Syslog *SaganProcSyslog = NULL;
 
-int proc_msgslot=0;
+int proc_msgslot = 0;
+
+unsigned char dynamic_rule_flag = 0;
+sbool reload_rules = false;
 
 pthread_cond_t SaganProcDoWork=PTHREAD_COND_INITIALIZER;
+
 pthread_mutex_t SaganProcWorkMutex=PTHREAD_MUTEX_INITIALIZER;
 pthread_mutex_t SaganMalformedCounter=PTHREAD_MUTEX_INITIALIZER;
+pthread_mutex_t SaganRulesLoadedMutex=PTHREAD_MUTEX_INITIALIZER;
+pthread_mutex_t SaganDynamicFlag=PTHREAD_MUTEX_INITIALIZER;
 
 /* ########################################################################
  * Start of main() thread
@@ -116,6 +130,8 @@ pthread_mutex_t SaganMalformedCounter=PTHREAD_MUTEX_INITIALIZER;
 int main(int argc, char **argv)
 {
 
+    (void)SetThreadName("SaganMain");
+
     const struct option long_options[] = {
         { "help",         no_argument,          NULL,   'h' },
         { "debug",        required_argument,    NULL,   'd' },
@@ -146,6 +162,15 @@ int main(int argc, char **argv)
     pthread_attr_setdetachstate(&thread_pcap_attr,  PTHREAD_CREATE_DETACHED);
 #endif
 
+    /****************************************************************************/
+    /* Redis local variables                                                    */
+    /****************************************************************************/
+
+#ifdef HAVE_LIBHIREDIS
+    char redis_reply[5];
+    char redis_command[300];
+#endif
+
     /****************************************************************************/
     /* Perfmonitor local variables                                              */
     /****************************************************************************/
@@ -183,9 +208,9 @@ int main(int argc, char **argv)
 
     struct sockaddr_in sa;
     char src_dns_lookup[20];
-    int  dns_flag=0;
 
-    sbool fifoerr=0;
+    sbool dns_flag = false;
+    sbool fifoerr = false;
 
     char *syslog_host=NULL;
     char *syslog_facility=NULL;
@@ -205,12 +230,12 @@ int main(int argc, char **argv)
 
     int i;
 
+    int dynamic_line_count = 0;
+
     time_t t;
     struct tm *run;
 
-    sbool debugflag=0;
-
-    /* Allocate and clear memory for global structs */
+    sbool debugflag = false;
 
     /* Allocate memory for global struct _SaganDebug */
 
@@ -276,7 +301,7 @@ int main(int argc, char **argv)
      */
 
     if ( !isatty(0) || !isatty(1) || !isatty(2) ) {
-        config->quiet = 1;
+        config->quiet = true;
     }
 
 
@@ -288,116 +313,127 @@ int main(int argc, char **argv)
             if (c == -1) break;
 
         case 'h':
-            Sagan_Usage();
+            Usage();
             exit(0);
             break;
 
         case 'Q':
-            config->quiet = 1;
+            config->quiet = true;
             break;
 
         case 'C':
-            Sagan_Credits();
+            Credits();
             exit(0);
             break;
 
         case 'd':
 
             if (Sagan_strstr(optarg, "malformed")) {
-                debug->debugmalformed=1;
-                debugflag=1;
+                debug->debugmalformed = true;
+                debugflag = true;
             }
 
             if (Sagan_strstr(optarg, "limits")) {
-                debug->debuglimits=1;
-                debugflag=1;
+                debug->debuglimits = true;
+                debugflag = true;
             }
 
             if (Sagan_strstr(optarg, "syslog")) {
-                debug->debugsyslog=1;
-                debugflag=1;
+                debug->debugsyslog = true;
+                debugflag = true;
             }
 
             if (Sagan_strstr(optarg, "load")) {
-                debug->debugload=1;
-                debugflag=1;
+                debug->debugload = true;
+                debugflag = true;
             }
 
             if (Sagan_strstr(optarg, "fwsam")) {
-                debug->debugfwsam=1;
-                debugflag=1;
+                debug->debugfwsam = true;
+                debugflag = true;
             }
 
             if (Sagan_strstr(optarg, "external")) {
-                debug->debugexternal=1;
-                debugflag=1;
+                debug->debugexternal = true;
+                debugflag = true;
             }
 
             if (Sagan_strstr(optarg, "threads")) {
-                debug->debugthreads=1;
-                debugflag=1;
+                debug->debugthreads = true;
+                debugflag = true;
             }
 
-            if (Sagan_strstr(optarg, "flowbit")) {
-                debug->debugflowbit=1;
-                debugflag=1;
+            if (Sagan_strstr(optarg, "xbit")) {
+                debug->debugxbit = true;
+                debugflag = true;
             }
 
             if (Sagan_strstr(optarg, "engine")) {
-                debug->debugengine=1;
-                debugflag=1;
+                debug->debugengine = true;
+                debugflag = true;
             }
 
             if (Sagan_strstr(optarg, "brointel")) {
-                debug->debugbrointel=1;
-                debugflag=1;
+                debug->debugbrointel = true;
+                debugflag = true;
             }
 
             if (Sagan_strstr(optarg, "ipc")) {
-                debug->debugipc=1;
-                debugflag=1;
+                debug->debugipc = true;
+                debugflag = true;
+            }
+
+            if (Sagan_strstr(optarg, "json")) {
+                debug->debugjson = true;
+                debugflag = true;
             }
 
 #ifdef HAVE_LIBMAXMINDDB
 
             if (Sagan_strstr(optarg, "geoip2")) {
-                debug->debuggeoip2=1;
-                debugflag=1;
+                debug->debuggeoip2 = true;
+                debugflag = true;
             }
 #endif
 
 #ifdef HAVE_LIBLOGNORM
             if (Sagan_strstr(optarg, "normalize" )) {
-                debug->debugnormalize=1;
-                debugflag=1;
+                debug->debugnormalize = true;
+                debugflag = true;
             }
 #endif
 
 #ifdef HAVE_LIBESMTP
             if (Sagan_strstr(optarg, "smtp")) {
-                debug->debugesmtp=1;
-                debugflag=1;
+                debug->debugesmtp = true;
+                debugflag = true;
             }
 #endif
 
 #ifdef HAVE_LIBPCAP
             if (Sagan_strstr(optarg, "plog")) {
-                debug->debugplog=1;
-                debugflag=1;
+                debug->debugplog = true;
+                debugflag = true;
             }
 #endif
 
 #ifdef WITH_BLUEDOT
             if (Sagan_strstr(optarg, "bluedot")) {
-                debug->debugbluedot=1;
-                debugflag=1;
+                debug->debugbluedot = true;
+                debugflag = true;
             }
 #endif
 
+#ifdef HAVE_LIBHIREDIS
+            if (Sagan_strstr(optarg, "redis")) {
+                debug->debugredis = true;
+                debugflag = true;
+            }
+#endif
 
             /* If option is unknown */
 
-            if ( debugflag == 0 ) {
+            if ( debugflag == false ) {
                 fprintf(stderr, "Unknown debug option %s!\n", optarg);
                 exit(1);
             }
@@ -406,7 +442,7 @@ int main(int argc, char **argv)
             break;
 
         case 'D':
-            config->daemonize=1;
+            config->daemonize = true;
             break;
 
         case 'u':
@@ -414,11 +450,11 @@ int main(int argc, char **argv)
             break;
 
         case 'c':
-            Sagan_Chroot(optarg);
+            Chroot(optarg);
             break;
 
         case 'F':
-            config->sagan_is_file=1;
+            config->sagan_is_file = true;
             strlcpy(config->sagan_fifo,optarg,sizeof(config->sagan_fifo) - 1);
             break;
 
@@ -432,15 +468,115 @@ int main(int argc, char **argv)
 
         default:
             fprintf(stderr, "Invalid argument! See below for command line switches.\n");
-            Sagan_Usage();
+            Usage();
             exit(0);
             break;
         }
     }
 
-    Sagan_Open_Log_File(OPEN, SAGAN_LOG);
+    Open_Log_File(OPEN, SAGAN_LOG);
 
-    Load_Config();
+    /* Become a daemon if requested */
+
+    if ( config->daemonize ) {
+
+        Sagan_Log(S_NORMAL, "Becoming a daemon!");
+
+        pid_t pid = 0;
+        pid = fork();
+
+        if ( pid == 0 ) {
+
+            /* Child */
+
+            if ( setsid() == -1 ) {
+                Sagan_Log(S_ERROR, "[%s, line %d] Failed creating new session while daemonizing", __FILE__, __LINE__);
+                exit(1);
+            }
+
+            pid = fork();
+
+            if ( pid == 0 ) {
+
+                /* Grandchild, the actual daemon */
+
+                if ( chdir("/") == -1 ) {
+                    Sagan_Log(S_ERROR, "[%s, line %d] Failed changing directory to / after daemonizing [errno %d]", __FILE__, __LINE__, errno);
+                    exit(1);
+                }
+
+                /* Close and re-open stdin, stdout, and stderr, so as to
+                   to release anyone waiting on them. */
+
+                close(0);
+                close(1);
+                close(2);
+
+                if ( open("/dev/null", O_RDONLY) == -1 ) {
+                    Sagan_Log(S_ERROR, "[%s, line %d] Failed reopening stdin after daemonizing [errno %d]", __FILE__, __LINE__, errno);
+                }
+
+                if ( open("/dev/null", O_WRONLY) == -1 ) {
+                    Sagan_Log(S_ERROR, "[%s, line %d] Failed reopening stdout after daemonizing [errno %d]", __FILE__, __LINE__, errno);
+                }
+
+                if ( open("/dev/null", O_RDWR) == -1 ) {
+                    Sagan_Log(S_ERROR, "[%s, line %d] Failed reopening stderr after daemonizing [errno %d]", __FILE__, __LINE__, errno);
+                }
+
+            } else if ( pid < 0 ) {
+
+                Sagan_Log(S_ERROR, "[%s, line %d] Failed second fork while daemonizing", __FILE__, __LINE__);
+                exit(1);
+
+            } else {
+
+                exit(0);
+            }
+
+        } else if ( pid < 0 ) {
+
+            Sagan_Log(S_ERROR, "[%s, line %d] Failed first fork while daemonizing", __FILE__, __LINE__);
+            exit(1);
+
+        } else {
+
+            /* Wait for child to exit */
+            waitpid(pid, NULL, 0);
+            exit(0);
+        }
+    }
+
+
+    /* Create the signal handlers thread _after_ the fork() so it can properly
+     * handly signals - Champ Clark III - 06/13/2011 */
+
+    rc = pthread_create( &sig_thread, NULL, (void *)Sig_Handler, NULL );
+
+    if ( rc != 0  ) {
+        Remove_Lock_File();
+        Sagan_Log(S_ERROR, "[%s, line %d] Error creating signal handler thread. [error: %d]", __FILE__, __LINE__, rc);
+    }
+
+
+#ifdef PCRE_HAVE_JIT
+
+    /* We test if pages will support RWX before loading rules.  If it doesn't due to the OS,
+       we want to disable PCRE JIT now.  This prevents confusing warnings of PCRE JIT during
+       rule load */
+
+    config->pcre_jit = true;
+
+    if (PageSupportsRWX() == false) {
+        Sagan_Log(S_WARN, "The operating system doens't allow RWX pages.  Disabling PCRE JIT.");
+        config->pcre_jit = false;
+    }
+
+#endif
+
+    pthread_mutex_lock(&SaganRulesLoadedMutex);
+    Load_YAML_Config(config->sagan_config);
+    pthread_mutex_unlock(&SaganRulesLoadedMutex);
 
     Sagan_Engine_Init();
 
@@ -457,9 +593,32 @@ int main(int argc, char **argv)
     pthread_attr_init(&thread_processor_attr);
     pthread_attr_setdetachstate(&thread_processor_attr,  PTHREAD_CREATE_DETACHED);
 
+#ifdef HAVE_LIBHIREDIS
+
+    /* Redis "writer" threads */
+
+    pthread_t redis_writer_processor_id[config->redis_max_writer_threads];
+    pthread_attr_t redis_writer_thread_processor_attr;
+    pthread_attr_init(&redis_writer_thread_processor_attr);
+    pthread_attr_setdetachstate(&redis_writer_thread_processor_attr,  PTHREAD_CREATE_DETACHED);
+
+#endif
+
     Sagan_Log(S_NORMAL, "Configuration file %s loaded and %d rules loaded.", config->sagan_config, counters->rulecount);
-    Sagan_Log(S_NORMAL, "Out of %d rules, %d Flowbit(s) are in use.", counters->rulecount, counters->flowbit_total_counter);
-    Sagan_Log(S_NORMAL, "Sagan version %s is firing up!", VERSION);
+    Sagan_Log(S_NORMAL, "Out of %d rules, %d xbit(s) are in use.", counters->rulecount, counters->xbit_total_counter);
+    Sagan_Log(S_NORMAL, "Out of %d rules, %d dynamic rule(s) are loaded.", counters->rulecount, counters->dynamic_rule_count);
+
+#ifdef PCRE_HAVE_JIT
+
+    if ( config->pcre_jit ) {
+        Sagan_Log(S_NORMAL, "PCRE JIT is enabled.");
+    }
+
+#endif
+
+    Sagan_Log(S_NORMAL, "");
+    Sagan_Log(S_NORMAL, "Sagan version %s is firing up on '%s'!", VERSION, config->sagan_sensor_name);
+    Sagan_Log(S_NORMAL, "");
 
 #ifdef HAVE_LIBPCAP
 
@@ -469,7 +628,7 @@ int main(int argc, char **argv)
 
     if ( config->plog_flag ) {
 
-        rc = pthread_create( &pcap_thread, NULL, (void *)plog_handler, NULL );
+        rc = pthread_create( &pcap_thread, NULL, (void *)Plog_Handler, NULL );
 
         if ( rc != 0 ) {
             Remove_Lock_File();
@@ -485,10 +644,10 @@ int main(int argc, char **argv)
 #endif
 
 
-    Sagan_Droppriv();              /* Become the Sagan user */
+    Droppriv();              /* Become the Sagan user */
     Sagan_Log(S_NORMAL, "---------------------------------------------------------------------------");
 
-    Sagan_IPC_Init();
+    IPC_Init();
 
     if ( config->perfmonitor_flag ) {
 
@@ -505,7 +664,7 @@ int main(int argc, char **argv)
 
     /* Open sagan alert file */
 
-    Sagan_Open_Log_File(OPEN, ALERT_LOG);
+    Open_Log_File(OPEN, ALERT_LOG);
 
     /****************************************************************************
      * Display processor information as we load
@@ -515,11 +674,11 @@ int main(int argc, char **argv)
 
     if ( config->sagan_track_clients_flag ) {
 
-        Sagan_Track_Clients_Init();
+        Track_Clients_Thread_Init();
 
         /* We run a thread for client_tracker_report */
 
-        rc = pthread_create( &ct_report_thread, NULL, (void *)Sagan_Report_Clients, NULL );
+        rc = pthread_create( &ct_report_thread, NULL, (void *)Track_Clients_Thread, NULL );
 
         if ( rc != 0 ) {
             Remove_Lock_File();
@@ -535,20 +694,20 @@ int main(int argc, char **argv)
 
     /* Sagan Blacklist IP processor *********************************************/
 
-    if ( config->blacklist_flag) {
+    if ( config->blacklist_flag ) {
+
         Sagan_Blacklist_Init();
         Sagan_Blacklist_Load();
+
     }
 
 #ifdef WITH_BLUEDOT
     if ( config->bluedot_flag ) {
 
         curl_global_init(CURL_GLOBAL_ALL);
-        Sagan_Bluedot_Init();
 
         Sagan_Log(S_NORMAL, "");
         Sagan_Log(S_NORMAL, "Bluedot URL: %s", config->bluedot_url);
-//            Sagan_Log(S_NORMAL, "Bluedot Auth: %s", config->bluedot_auth);
         Sagan_Log(S_NORMAL, "Bluedot Device ID: %s", config->bluedot_device_id);
         Sagan_Log(S_NORMAL, "Bluedot Categories File: %s", config->bluedot_cat);
         Sagan_Log(S_NORMAL, "Bluedot Max Cache: %d", config->bluedot_max_cache);
@@ -588,30 +747,32 @@ int main(int argc, char **argv)
      ***************************************************************************/
 
 #ifdef HAVE_LIBESMTP
+
     if ( config->sagan_esmtp_flag ) {
         Sagan_Log(S_NORMAL, "");
 
-        if ( config->min_email_priority ) {
-            Sagan_Log(S_NORMAL, "E-mail on priority %d or higher.", config->min_email_priority);
-        }
-
         Sagan_Log(S_NORMAL, "E-Mail will be sent from: %s", config->sagan_esmtp_from);
         Sagan_Log(S_NORMAL, "SMTP server is set to: %s", config->sagan_esmtp_server);
     }
+
 #endif
 
 #ifdef WITH_SNORTSAM
 
     if ( config->sagan_fwsam_flag ) {
+
         Sagan_Log(S_NORMAL, "");
         Sagan_Log(S_NORMAL, "Snortsam output plug in enabled.");
+
     }
 
 #endif
 
     if ( config->sagan_external_output_flag ) {
+
         Sagan_Log(S_NORMAL, "");
-        Sagan_Log(S_NORMAL, "External program to be called: %s", config->sagan_extern);
+        Sagan_Log(S_NORMAL, "External program to be called: %s", config->sagan_external_command);
+
     }
 
     /* Unified2 ****************************************************************/
@@ -619,10 +780,12 @@ int main(int argc, char **argv)
 #if defined(HAVE_DNET_H) || defined(HAVE_DUMBNET_H)
 
     if ( config->sagan_unified2_flag ) {
+
         Sagan_Log(S_NORMAL, "");
         Sagan_Log(S_NORMAL, "Unified2 file: %s", config->unified2_filepath);
         Sagan_Log(S_NORMAL, "Unified2 limit: %dM", config->unified2_limit  / 1024 / 1024 );
         Unified2InitFile();
+
     }
 
 #endif
@@ -635,9 +798,11 @@ int main(int argc, char **argv)
     /* What to "ignore" ********************************************************/
 
     if ( config->sagan_droplist_flag ) {
+
         Load_Ignore_List();
         Sagan_Log(S_NORMAL, "");
         Sagan_Log(S_NORMAL, "Loaded %d ignore/drop list item(s).", counters->droplist_count);
+
     }
 
     /***************************************************************************
@@ -648,50 +813,27 @@ int main(int argc, char **argv)
     Sagan_Log(S_NORMAL, " ,-._,-. 	-*> Sagan! <*-");
     Sagan_Log(S_NORMAL, " \\/)\"(\\/	Version %s", VERSION);
     Sagan_Log(S_NORMAL, "  (_o_)	Champ Clark III & The Quadrant InfoSec Team [quadrantsec.com]");
-    Sagan_Log(S_NORMAL, "  /   \\/)	Copyright (C) 2009-2016 Quadrant Information Security, et al.");
+    Sagan_Log(S_NORMAL, "  /   \\/)	Copyright (C) 2009-2017 Quadrant Information Security, et al.");
     Sagan_Log(S_NORMAL, " (|| ||) 	Using PCRE version: %s", pcre_version());
     Sagan_Log(S_NORMAL, "  oo-oo     Sagan is processing events.....");
     Sagan_Log(S_NORMAL, "");
 
-    /* Become a daemon if requested */
 
-    if ( config->daemonize ) {
-        Sagan_Log(S_NORMAL, "Becoming a daemon!");
-
-        pid_t pid = 0;
-        setsid();
-        pid = fork();
-        if (pid == 0) {}
-        else {
-            exit(0);
-        }
-    }
-
-
-    /* Create the signal handlers thread _after_ the fork() so it can properly
-     * handly signals - Champ Clark III - 06/13/2011 */
-
-    rc = pthread_create( &sig_thread, NULL, (void *)Sig_Handler, NULL );
-
-    if ( rc != 0  ) {
-        Remove_Lock_File();
-        Sagan_Log(S_ERROR, "[%s, line %d] Error creating signal handler thread. [error: %d]", __FILE__, __LINE__, rc);
-    }
-
-
-    /* We don't want the key_handler() if we're in daemon mode! */
+    /* We don't want the Key_Handler() if we're in daemon mode! */
 
     if (!config->daemonize ) {
+
         if (!config->quiet) {
 
-            rc = pthread_create( &key_thread, NULL, (void *)key_handler, NULL );
+            rc = pthread_create( &key_thread, NULL, (void *)Key_Handler, NULL );
 
             if ( rc != 0 ) {
+
                 Remove_Lock_File();
-                Sagan_Log(S_ERROR, "[%s, line %d] Error creating key_handler thread. [error: %d]", __FILE__, __LINE__, rc);
+                Sagan_Log(S_ERROR, "[%s, line %d] Error creating Key_Handler() thread. [error: %d]", __FILE__, __LINE__, rc);
+
             }
         }
-
     }
 
     /* We do this after forking so init scripts can complete */
@@ -699,37 +841,100 @@ int main(int argc, char **argv)
     /* Check lock file _after_ thread.  If you don't it'll retreive the wrong pid
      * and incorrectly believe there is a stale lock file if --daemon */
 
-    checklockfile();
+    CheckLockFile();
+
+#ifdef HAVE_LIBHIREDIS
+
+    /* Right now,  Redis is only used for xbit storage */
+
+    if ( config->redis_flag && config->xbit_storage == XBIT_STORAGE_REDIS ) {
+
+        Redis_Writer_Init();
+        Redis_Reader_Connect();
+
+        if ( config->redis_password[0] != '\0' ) {
+
+            snprintf(redis_command, sizeof(redis_command), "AUTH %s", config->redis_password);
+            Redis_Reader(redis_command, redis_reply, sizeof(redis_reply));
+
+            if (!strcmp(redis_reply, "OK")) {
+                Sagan_Log(S_NORMAL, "Authentication success for 'reader' to Redis server at %s:%d.", config->redis_server, config->redis_port);
+            } else {
+
+                Remove_Lock_File();
+                Sagan_Log(S_ERROR, "Authentication failure for 'reader' to Redis server at %s:%d. Abort!", config->redis_server, config->redis_port);
+            }
+        }
+
+        strlcpy(redis_command, "PING", sizeof(redis_command));
+        Redis_Reader(redis_command, redis_reply, sizeof(redis_reply));
+
+        if (!strcmp(redis_reply, "PONG")) {
+            Sagan_Log(S_NORMAL, "Got 'reader' PONG from Redis at %s:%d.", config->redis_server, config->redis_port);
+        }
+
+        Sagan_Log(S_NORMAL, "");
+
+    }
+
+#endif
 
     Sagan_Log(S_NORMAL, "Spawning %d Processor Threads.", config->max_processor_threads);
 
     for (i = 0; i < config->max_processor_threads; i++) {
 
-        rc = pthread_create ( &processor_id[i], &thread_processor_attr, (void *)Sagan_Processor, NULL );
+        rc = pthread_create ( &processor_id[i], &thread_processor_attr, (void *)Processor, NULL );
 
         if ( rc != 0 ) {
+
             Remove_Lock_File();
             Sagan_Log(S_ERROR, "Could not pthread_create() for I/O processors [error: %d]", rc);
+
         }
     }
 
+#ifdef HAVE_LIBHIREDIS
+
+    if ( config->redis_flag && config->xbit_storage == XBIT_STORAGE_REDIS ) {
+
+        Sagan_Log(S_NORMAL, "Spawning %d Redis Writer Threads.", config->redis_max_writer_threads);
+
+        for (i = 0; i < config->redis_max_writer_threads; i++) {
+
+            rc = pthread_create ( &redis_writer_processor_id[i], &redis_writer_thread_processor_attr, (void *)Redis_Writer, NULL );
+
+            if ( rc != 0 ) {
+
+                Remove_Lock_File();
+                Sagan_Log(S_ERROR, "Could not pthread_create() for I/O redis writers [error: %d]", rc);
+
+            }
+        }
+    }
+
+#endif
+
     Sagan_Log(S_NORMAL, "");
 
-    if ( config->sagan_is_file == 0 ) {
+    if ( !config->sagan_is_file ) {
+
         Sagan_Log(S_NORMAL, "Attempting to open syslog FIFO (%s).", config->sagan_fifo);
+
     } else {
+
         Sagan_Log(S_NORMAL, "Attempting to open syslog FILE (%s).", config->sagan_fifo);
+
     }
 
 
 
-    while(1) {
+    while(true) {
 
         FILE *fd;
 
         if (( fd = fopen(config->sagan_fifo, "r" )) == NULL ) {
 
-            if ( config->sagan_is_file == 0 ) {
+            if ( config->sagan_is_file == false ) {
 
                 /* try to create it */
 
@@ -752,12 +957,12 @@ int main(int argc, char **argv)
 
         }
 
-        if ( config->sagan_is_file == 0 ) {
+        if ( config->sagan_is_file == false ) {
             Sagan_Log(S_NORMAL, "Successfully opened FIFO (%s).", config->sagan_fifo);
 
-#if defined(F_GETPIPE_SZ) && defined(F_SETPIPE_SZ)
+#if defined(HAVE_GETPIPE_SZ) && defined(HAVE_SETPIPE_SZ)
 
-            Sagan_Set_Pipe_Size(fd);
+            Set_Pipe_Size(fd);
 
 #endif
 
@@ -772,20 +977,27 @@ int main(int argc, char **argv)
 
                 /* If the FIFO was in a error state,  let user know the FIFO writer has resumed */
 
-                if ( fifoerr == 1 ) {
+                if ( fifoerr == true ) {
+
                     Sagan_Log(S_NORMAL, "FIFO writer has restarted. Processing events.");
 
-#if defined(F_GETPIPE_SZ) && defined(F_SETPIPE_SZ)
+#if defined(HAVE_GETPIPE_SZ) && defined(HAVE_SETPIPE_SZ)
 
-                    Sagan_Set_Pipe_Size(fd);
+                    Set_Pipe_Size(fd);
 
 #endif
-
-                    fifoerr=0;
+                    fifoerr = false;
                 }
 
                 counters->sagantotal++;
 
+                /* If Dynamic rules are loaded,  keep track of line count */
+
+                if ( config->dynamic_load_flag == true ) {
+
+                    dynamic_line_count++;
+                }
+
                 syslog_host = strtok_r(syslogstring, "|", &tok);
 
                 /* If we're using DNS (and we shouldn't be!),  we start DNS checks and lookups
@@ -794,27 +1006,30 @@ int main(int argc, char **argv)
 
                 if (config->syslog_src_lookup ) {
                     if ( inet_pton(AF_INET, syslog_host, &(sa.sin_addr)) == 0 ) { 	/* Is inbound a valid IP? */
-                        dns_flag=0;
+                        dns_flag = false;
 
                         for(i=0; i <= counters->dns_cache_count ; i++) {			/* Check cache first */
                             if (!strcmp( dnscache[i].hostname, syslog_host)) {
                                 syslog_host = dnscache[i].src_ip;
-                                dns_flag=1;
+                                dns_flag = true;
                             }
                         }
 
                         /* If entry was not found in cache,  look it up */
 
-                        if ( dns_flag == 0 ) {
+                        if ( dns_flag == false ) {
 
                             /* Do a DNS lookup */
-                            strlcpy(src_dns_lookup, DNS_Lookup(syslog_host), sizeof(src_dns_lookup));
+
+                            rc = DNS_Lookup(syslog_host, src_dns_lookup, sizeof(src_dns_lookup));
 
                             /* Invalid lookups get the config->sagan_host value */
 
-                            if (src_dns_lookup[0] == '0' ) {
+                            if ( rc == -1 ) {
+
                                 strlcpy(src_dns_lookup, config->sagan_host, sizeof(src_dns_lookup));
                                 counters->dns_miss_count++;
+
                             }
 
 
@@ -823,7 +1038,9 @@ int main(int argc, char **argv)
                             dnscache = (_SaganDNSCache *) realloc(dnscache, (counters->dns_cache_count+1) * sizeof(_SaganDNSCache));
 
                             if ( dnscache == NULL ) {
+
                                 Sagan_Log(S_ERROR, "[%s, line %d] Failed to reallocate memory for dnscache. Abort!", __FILE__, __LINE__);
+
                             }
 
                             strlcpy(dnscache[counters->dns_cache_count].hostname, syslog_host, sizeof(dnscache[counters->dns_cache_count].hostname));
@@ -849,14 +1066,15 @@ int main(int argc, char **argv)
                         if ( debug->debugmalformed ) {
                             Sagan_Log(S_WARN, "Sagan received a malformed 'host' (replaced with %s)", config->sagan_host);
                         }
-
                     }
                 }
 
                 /* We now check the rest of the values */
 
                 syslog_facility=strtok_r(NULL, "|", &tok);
+
                 if ( syslog_facility == NULL ) {
+
                     syslog_facility = "SAGAN: FACILITY ERROR";
 
                     pthread_mutex_lock(&SaganMalformedCounter);
@@ -869,7 +1087,9 @@ int main(int argc, char **argv)
                 }
 
                 syslog_priority=strtok_r(NULL, "|", &tok);
+
                 if ( syslog_priority == NULL ) {
+
                     syslog_priority = "SAGAN: PRIORITY ERROR";
 
                     pthread_mutex_lock(&SaganMalformedCounter);
@@ -882,7 +1102,9 @@ int main(int argc, char **argv)
                 }
 
                 syslog_level=strtok_r(NULL, "|", &tok);
+
                 if ( syslog_level == NULL ) {
+
                     syslog_level = "SAGAN: LEVEL ERROR";
 
                     pthread_mutex_lock(&SaganMalformedCounter);
@@ -895,7 +1117,9 @@ int main(int argc, char **argv)
                 }
 
                 syslog_tag=strtok_r(NULL, "|", &tok);
+
                 if ( syslog_tag == NULL ) {
+
                     syslog_tag = "SAGAN: TAG ERROR";
 
                     pthread_mutex_lock(&SaganMalformedCounter);
@@ -908,7 +1132,9 @@ int main(int argc, char **argv)
                 }
 
                 syslog_date=strtok_r(NULL, "|", &tok);
+
                 if ( syslog_date == NULL ) {
+
                     syslog_date = "SAGAN: DATE ERROR";
 
                     pthread_mutex_lock(&SaganMalformedCounter);
@@ -921,7 +1147,9 @@ int main(int argc, char **argv)
                 }
 
                 syslog_time=strtok_r(NULL, "|", &tok);
+
                 if ( syslog_time == NULL ) {
+
                     syslog_time = "SAGAN: TIME ERROR";
 
                     pthread_mutex_lock(&SaganMalformedCounter);
@@ -935,7 +1163,9 @@ int main(int argc, char **argv)
 
 
                 syslog_program=strtok_r(NULL, "|", &tok);
+
                 if ( syslog_program == NULL ) {
+
                     syslog_program = "SAGAN: PROGRAM ERROR";
 
                     pthread_mutex_lock(&SaganMalformedCounter);
@@ -948,7 +1178,9 @@ int main(int argc, char **argv)
                 }
 
                 syslog_msg=strtok_r(NULL, "", &tok);		/* In case the message has | in it,  we delimit on "" */
+
                 if ( syslog_msg == NULL ) {
+
                     syslog_msg = "SAGAN: MESSAGE ERROR";
 
                     pthread_mutex_lock(&SaganMalformedCounter);
@@ -974,12 +1206,6 @@ int main(int argc, char **argv)
                 }
 
 
-                /*
-                	                    if ( config->sagan_track_clients_flag )
-                       		                {
-                                            	Sagan_Track_Clients( IP2Bit(syslog_host) );
-                                        	}
-                */
                 if ( proc_msgslot < config->max_processor_threads ) {
 
                     pthread_mutex_lock(&SaganProcWorkMutex);
@@ -994,17 +1220,39 @@ int main(int argc, char **argv)
                     strlcpy(SaganProcSyslog[proc_msgslot].syslog_program, syslog_program, sizeof(SaganProcSyslog[proc_msgslot].syslog_program));
                     strlcpy(SaganProcSyslog[proc_msgslot].syslog_message, syslog_msg, sizeof(SaganProcSyslog[proc_msgslot].syslog_message));
 
+                    if ( config->dynamic_load_flag == true && ( dynamic_line_count >= config->dynamic_load_sample_rate ) ) {
+
+                        pthread_mutex_lock(&SaganDynamicFlag);
+                        dynamic_rule_flag = DYNAMIC_RULE;
+                        pthread_mutex_unlock(&SaganDynamicFlag);
+
+                        dynamic_line_count = 0;
+                    }
+
+
+                    /* Thread holds here if rule load is in progress */
+
+                    if ( config->dynamic_load_flag == true ) {
+
+                        pthread_mutex_lock(&SaganRulesLoadedMutex);
+                        reload_rules = true;
+                        pthread_mutex_unlock(&SaganRulesLoadedMutex);
+
+                    }
+
                     proc_msgslot++;
 
                     pthread_cond_signal(&SaganProcDoWork);
                     pthread_mutex_unlock(&SaganProcWorkMutex);
+
                 } else {
+
                     counters->worker_thread_exhaustion++;
                     counters->sagan_log_drop++;
                 }
 
                 if (debug->debugthreads) {
-                    Sagan_Log(S_DEBUG, "Current \"proc_msgslot\": %d", proc_msgslot);
+                    Sagan_Log(S_DEBUG, "[%s, line %d] Current \"proc_msgslot\": %d", __FILE__, __LINE__, proc_msgslot);
                 }
 
                 if (debug->debugsyslog) {
@@ -1020,10 +1268,10 @@ int main(int argc, char **argv)
 
             /* fgets() has returned a error,  likely due to the FIFO writer leaving */
 
-            /* DEBUG : set a kill flag and join */
             /* RMEOVE LOCK */
 
-            if ( fifoerr == 0 ) {
+            if ( fifoerr == false ) {
+
                 if ( config->sagan_is_file != 0 ) {
                     Sagan_Log(S_NORMAL, "EOF reached. Waiting for threads to catch up....");
                     Sagan_Log(S_NORMAL, "");
@@ -1034,14 +1282,16 @@ int main(int argc, char **argv)
                     }
 
                     fclose(fd);
-                    Sagan_Statistics();
+                    Statistics();
                     Remove_Lock_File();
 
-                    Sagan_Log(S_NORMAL, "Exiting.");		/* DEBUG: Rejoin threads */
+                    Sagan_Log(S_NORMAL, "Exiting.");
                     exit(0);
+
                 } else {
+
                     Sagan_Log(S_WARN, "FIFO writer closed.  Waiting for FIFO writer to restart....");
-                    fifoerr=1; 			/* Set flag so our wile(fgets) knows */
+                    fifoerr = true; 			/* Set flag so our wile(fgets) knows */
                 }
             }
             sleep(1);		/* So we don't eat 100% CPU */
