@@ -1,6 +1,6 @@
 /*
-** Copyright (C) 2009-2016 Quadrant Information Security <quadrantsec.com>
-** Copyright (C) 2009-2016 Champ Clark III <cclark@quadrantsec.com>
+** Copyright (C) 2009-2017 Quadrant Information Security <quadrantsec.com>
+** Copyright (C) 2009-2017 Champ Clark III <cclark@quadrantsec.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License Version 2 as
@@ -26,7 +26,7 @@
  *
  */
 
-/* DEBUG: need to add dstport, srcport for threshold/after */
+/* TODO: need to add dstport, srcport for threshold/after */
 
 
 #include <stdio.h>
@@ -44,9 +44,9 @@
 
 #include "../src/sagan.h"
 #include "../src/sagan-defs.h"
-#include "../src/sagan-flowbit.h"
+#include "../src/xbit-mmap.h"
 
-#include "../src/processors/sagan-track-clients.h"
+#include "../src/processors/track-clients.h"
 
 /****************************************************************************
  * usage - Give the user some hints about how to use this utility!
@@ -110,7 +110,7 @@ int main(int argc, char **argv)
 
     struct _Sagan_IPC_Counters *counters_ipc;
 
-    struct _Sagan_IPC_Flowbit *flowbit_ipc;
+    struct _Sagan_IPC_Xbit *xbit_ipc;
     struct _Sagan_Track_Clients_IPC *SaganTrackClients_ipc;
 
     struct thresh_by_src_ipc *threshbysrc_ipc;
@@ -283,7 +283,7 @@ int main(int argc, char **argv)
 
 
         for ( i = 0; i < counters_ipc->thresh_count_by_username; i++) {
-            printf("%-16s| %-15d| %-21s| %-11s| %d\n", inet_ntoa(ip_addr_src), threshbyusername_ipc[i].count, u32_time_to_human(threshbyusername_ipc[i].utime), threshbyusername_ipc[i].sid, threshbyusername_ipc[i].expire);
+            printf("%-16s| %-15d| %-21s| %-11s| %d\n", threshbyusername_ipc[i].username, threshbyusername_ipc[i].count, u32_time_to_human(threshbyusername_ipc[i].utime), threshbyusername_ipc[i].sid, threshbyusername_ipc[i].expire);
         }
     }
 
@@ -312,13 +312,13 @@ int main(int argc, char **argv)
     if ( counters_ipc->after_count_by_src >= 1 ) {
 
         printf("\n***  After by source (%d) ***\n", counters_ipc->after_count_by_src);
-        printf("---------------------------------------------------------------------------------\n");
-        printf("%-16s| %-15s| %-21s| %-11s| %s\n", "SRC IP", "Counter","Date added/modified", "SID", "Expire" );
-        printf("---------------------------------------------------------------------------------\n");
+        printf("-----------------------------------------------------------------------------------------------------\n");
+        printf("%-16s| %-15s| %-21s| %-15s| %-11s| %s\n", "SRC IP", "Counter","Timestamp added", "Total", "SID", "Expire" );
+        printf("-----------------------------------------------------------------------------------------------------\n");
 
         for ( i = 0; i < counters_ipc->after_count_by_src; i++) {
             ip_addr_src.s_addr = htonl(afterbysrc_ipc[i].ipsrc);
-            printf("%-16s| %-15d| %-21s| %-11s| %d\n", inet_ntoa(ip_addr_src), afterbysrc_ipc[i].count, u32_time_to_human(afterbysrc_ipc[i].utime), afterbysrc_ipc[i].sid, afterbysrc_ipc[i].expire);
+            printf("%-16s| %-15d| %-21s| %-15d| %-11s| %d\n", inet_ntoa(ip_addr_src), afterbysrc_ipc[i].count, u32_time_to_human(afterbysrc_ipc[i].utime), afterbysrc_ipc[i].total_count, afterbysrc_ipc[i].sid, afterbysrc_ipc[i].expire);
         }
     }
 
@@ -348,14 +348,14 @@ int main(int argc, char **argv)
     if ( counters_ipc->after_count_by_dst >= 1 ) {
 
         printf("\n***  After by destination (%d)***\n", counters_ipc->after_count_by_dst);
-        printf("---------------------------------------------------------------------------------\n");
-        printf("%-16s| %-15s| %-21s| %-11s| %s\n", "DST IP", "Counter","Date added/modified", "SID", "Expire" );
-        printf("---------------------------------------------------------------------------------\n");
+        printf("-----------------------------------------------------------------------------------------------------\n");
+        printf("%-16s| %-15s| %-21s| %-15s| %-11s| %s\n", "DST IP", "Counter","Timestamp added", "Total", "SID", "Expire" );
+        printf("-----------------------------------------------------------------------------------------------------\n");
 
         for ( i = 0; i < counters_ipc->after_count_by_dst; i++) {
             ip_addr_dst.s_addr = htonl(afterbydst_ipc[i].ipdst);
 
-            printf("%-16s| %-15d| %-21s| %-11s| %d\n", inet_ntoa(ip_addr_dst), afterbydst_ipc[i].count, u32_time_to_human(afterbydst_ipc[i].utime), afterbydst_ipc[i].sid, afterbydst_ipc[i].expire);
+            printf("%-16s| %-15d| %-21s| %-15d| %-11s| %d\n", inet_ntoa(ip_addr_dst), afterbydst_ipc[i].count, u32_time_to_human(afterbydst_ipc[i].utime), afterbydst_ipc[i].total_count, afterbydst_ipc[i].sid, afterbydst_ipc[i].expire);
         }
     }
 
@@ -385,20 +385,21 @@ int main(int argc, char **argv)
     if ( counters_ipc->after_count_by_username >= 1 ) {
 
         printf("\n***  After by username ***(%d)\n", counters_ipc->after_count_by_username);
-        printf("---------------------------------------------------------------------------------\n");
-        printf("%-16s| %-15s| %-21s| %-11s| %s\n", "Username", "Counter","Date added/modified", "SID", "Expire" );
-        printf("---------------------------------------------------------------------------------\n");
+        printf("-----------------------------------------------------------------------------------------------------\n");
+        printf("%-16s| %-15s| %-21s| %-15s| %-11s| %s\n", "Username", "Counter","Timestamp added", "Total", "SID", "Expire" );
+        printf("-----------------------------------------------------------------------------------------------------\n");
+
 
 
 
         for ( i = 0; i < counters_ipc->after_count_by_username; i++) {
-            printf("%-16s| %-15d| %-21s| %-11s| %d\n", afterbyusername_ipc[i].username, afterbyusername_ipc[i].count, u32_time_to_human(afterbyusername_ipc[i].utime), afterbyusername_ipc[i].sid, afterbyusername_ipc[i].expire);
+            printf("%-16s| %-15d| %-21s| %-15d| %-11s| %d\n", afterbyusername_ipc[i].username, afterbyusername_ipc[i].count, u32_time_to_human(afterbyusername_ipc[i].utime), afterbyusername_ipc[i].total_count, afterbyusername_ipc[i].sid, afterbyusername_ipc[i].expire);
         }
     }
 
-    /*** Get "flowbit" data ***/
+    /*** Get "xbit" data ***/
 
-    snprintf(tmp_object_check, sizeof(tmp_object_check) - 1, "%s/%s", ipc_directory, FLOWBIT_IPC_FILE);
+    snprintf(tmp_object_check, sizeof(tmp_object_check) - 1, "%s/%s", ipc_directory, XBIT_IPC_FILE);
 
     if ( object_check(tmp_object_check) == false ) {
         fprintf(stderr, "Error.  Can't locate %s. Abort!\n", tmp_object_check);
@@ -411,7 +412,7 @@ int main(int argc, char **argv)
         exit(1);
     }
 
-    if (( flowbit_ipc = mmap(0, sizeof(_Sagan_IPC_Flowbit) + (sizeof(_Sagan_IPC_Flowbit) * counters_ipc->flowbit_count ) , PROT_READ, MAP_SHARED, shm, 0)) == MAP_FAILED ) {
+    if (( xbit_ipc = mmap(0, sizeof(_Sagan_IPC_Xbit) + (sizeof(_Sagan_IPC_Xbit) * counters_ipc->xbit_count ) , PROT_READ, MAP_SHARED, shm, 0)) == MAP_FAILED ) {
         fprintf(stderr, "[%s, line %d] Error allocating memory object! [%s]\n", __FILE__, __LINE__, strerror(errno));
         exit(1);
     }
@@ -419,29 +420,31 @@ int main(int argc, char **argv)
     close(shm);
 
 
-    if ( counters_ipc->flowbit_count >= 1 ) {
+    if ( counters_ipc->xbit_count >= 1 ) {
 
-        printf("\n*** Flowbits (%d) ****\n", counters_ipc->flowbit_count);
-        printf("-----------------------------------------------------------------------------------------------------------------------------\n");
-        printf("%-9s| %-25s| %-16s| %-16s| %-21s| %s\n", "S", "Flowbit name", "SRC IP", "DST IP", "Date added/modified", "Expire");
-        printf("-----------------------------------------------------------------------------------------------------------------------------\n");
+        printf("\n*** Xbits (%d) ****\n", counters_ipc->xbit_count);
+        printf("----------------------------------------------------------------------------------------------------------------------------------------------\n");
+        printf("%-9s| %-25s| %-16s| %-16s| %-8s| %-8s| %-21s| %s\n", "S", "Xbit name", "SRC IP", "DST IP", "SRC PRT", "DST PRT", "Date added/modified", "Expire");
+        printf("----------------------------------------------------------------------------------------------------------------------------------------------\n");
 
-        for ( i = 0; i < counters_ipc->flowbit_count; i++) {
+        for ( i = 0; i < counters_ipc->xbit_count; i++) {
 
-            ip_addr_src.s_addr = htonl(flowbit_ipc[i].ip_src);
-            ip_addr_dst.s_addr = htonl(flowbit_ipc[i].ip_dst);
+            ip_addr_src.s_addr = htonl(xbit_ipc[i].ip_src);
+            ip_addr_dst.s_addr = htonl(xbit_ipc[i].ip_dst);
 
-            if ( flowbit_ipc[i].flowbit_state == 1 ) {
+            if ( xbit_ipc[i].xbit_state == 1 ) {
 
-                printf("ACTIVE   | %-25s| ", flowbit_ipc[i].flowbit_name);
+                printf("ACTIVE   | %-25s| ", xbit_ipc[i].xbit_name);
             } else {
-                printf("INACTIVE | %-25s| ", flowbit_ipc[i].flowbit_name);
+                printf("INACTIVE | %-25s| ", xbit_ipc[i].xbit_name);
             }
 
             printf("%-16s| ", inet_ntoa(ip_addr_src));
             printf("%-16s| ", inet_ntoa(ip_addr_dst));
-            printf("%-21s| ", u32_time_to_human(flowbit_ipc[i].flowbit_date));
-            printf("%d (%s)\n", flowbit_ipc[i].expire, u32_time_to_human(flowbit_ipc[i].flowbit_date + flowbit_ipc[i].expire));
+            printf("%-8d| ", xbit_ipc[i].src_port);
+            printf("%-8d| ", xbit_ipc[i].dst_port);
+            printf("%-21s| ", u32_time_to_human(xbit_ipc[i].xbit_date));
+            printf("%d (%s)\n", xbit_ipc[i].expire, u32_time_to_human(xbit_ipc[i].xbit_date + xbit_ipc[i].expire));
 
         }
     }
